require 'rest-client'
require 'json'
require 'rubygems'
require 'zip'

class InvoiceController < ApplicationController
  include InvoiceHelper
  skip_before_action :verify_authenticity_token
  before_action :authenticate_user!
  def index
  	@invoices       = Invoice.all.order(:folio)
    @missing_folios = (1..(@invoices.last.folio.to_i)).to_a
    @invoices.each do |invoice|
      @missing_folios.delete invoice.folio.to_i
    end
  end

  def init_aws
    Aws.config.update({
      region: ENV['AWS_REGION'],
      credentials: Aws::Credentials.new(ENV['AWS_ID'], ENV['AWS_SECRET'])
    })
    return Aws::S3::Client.new
  end

  def info4pdf(id)
    invoice     = Invoice.find id
    debtor      = invoice.debtor
    metodo_pago = MetodoPago.find invoice.payment_method
    forma_pago  = Formapago.find invoice.payment_form
    uso_cfdi    = Uso.find invoice.cfdi_use
    return {
      :xml           => invoice.xml, 
      :cadena        => invoice.cadena_original,
      :id            => invoice.id,
      :nombre        => debtor.name,
      :cuenta        => debtor.cuenta,
      :debtorno      => debtor.debtorno,
      :metodo_pago   => metodo_pago.descripcion,
      :forma_pago    => forma_pago.descripcion,
      :uso_cfdi      => uso_cfdi.descripcion,
      :uuid          => invoice.uuid,
      :occliente     => invoice.occliente,
      :observaciones => invoice.observaciones,
      :status        => invoice.invoice_status,
      :direccion     => "#{debtor.address1} #{debtor.address2}\n#{debtor.address4} #{debtor.address7}\n#{debtor.address8} #{debtor.address9}\n#{debtor.address10}"
    }
  end

  def download_xml
  	invoice = Invoice.find params[:id]
  	send_data invoice.xml, filename: "#{invoice.uuid}.xml", type: "application/xml"
  end

  def download_pdf
  	data = info4pdf params[:id]
    logger.debug data.to_json
    pdf = RestClient.post ENV['API_HOST']+'/pdf/create', data.to_json, {content_type: :json}
  	send_data pdf, filename: "#{data[:uuid]}.pdf", type: "application/pdf"
  end

  def masive_download
    ids = params[:ids]
    t = Tempfile.new "ziptemp#{Time.now}"
    Zip::File.open(t.path, Zip::File::CREATE) do |zipfile|
      ids.each do |id|
        invoice = Invoice.find id
        zipfile.get_output_stream(invoice.uuid + ".xml") { |f|
          f.write invoice.xml
        }
        info = info4pdf id
        pdf  = RestClient.post ENV['API_HOST']+'/pdf/create', info.to_json, {content_type: :json}
        zipfile.get_output_stream(invoice.uuid + ".pdf") { |f|
          f.write pdf
        }
      end
    end
    puts t.path
    s3 = init_aws
    zipname = "facturas#{Time.now}.zip"
    s3.put_object(
      :acl => "public-read", 
      :bucket => "gasidealassets",
      :key => "zipfactura/"+zipname,
      :body => IO.read(t.path)
    )
    render json: {:link => "https://s3.amazonaws.com/gasidealassets/zipfactura/"+zipname}
  end

  def activate
    invoice = Invoice.find params[:id]
    invoice.invoice_status = 'active'
    invoice.save
    render json: {id: params[:id]}
  end

  def cancel
    invoice_id = params[:id]
    response = RestClient.post ENV['API_HOST']+'/invoice/cancel', {id: invoice_id}.to_json, {content_type: :json}
    puts response
    json = JSON.parse response.body
    puts json
    if json["status"] != 200
      render json: {status: json["status"]}
    end
    invoice = Invoice.find invoice_id
    invoice.invoice_status = 'cancelled'
    invoice.save
    render json: {status: 200, id: invoice_id}
  end

  def pay
    invoice = Invoice.find params[:id]
    invoice.paid_status = 'paid'
    invoice.save
    flash[:notice] = "Factura pagada correctamente."
    redirect_to action: 'index'
  end

  def cancellpayment
    invoice = Invoice.find params[:id]
    invoice.paid_status = 'pending'
    invoice.save
    flash[:notice] = "Se ha cancelado el pago de la factura."
    redirect_to action: 'index'
  end

  def mail
    invoice     = Invoice.find params[:id]
    debtor      = invoice.debtor
    metodo_pago = MetodoPago.find invoice.payment_method
    forma_pago  = Formapago.find invoice.payment_form
    uso_cfdi    = Uso.find invoice.cfdi_use
    data = {
      :xml           => invoice.xml,
      :uuid          => invoice.uuid,
      :cadena        => invoice.cadena_original,
      :id            => invoice.id,
      :nombre        => debtor.name,
      :cuenta        => debtor.cuenta,
      :debtorno      => debtor.debtorno,
      :correo        => debtor.email,
      :metodo_pago   => metodo_pago.descripcion,
      :forma_pago    => forma_pago.descripcion,
      :uso_cfdi      => uso_cfdi.descripcion,
      :direccion     => "#{debtor.address1} #{debtor.address2}\n#{debtor.address4} #{debtor.address7}\n#{debtor.address8} #{debtor.address9}\n#{debtor.address10}",
      :occliente     => invoice.occliente,
      :status        => invoice.invoice_status,
      :observaciones => invoice.observaciones
    }.to_json
    puts data
    response = RestClient.post ENV['API_HOST']+'/invoice/mail', data, {content_type: :json}
    render json: (JSON.parse response.body)
  end


  def show
    @invoice   = Invoice.find params[:id]
    doc        = Nokogiri::XML(@invoice.xml)
    @conceptos = doc.xpath("//cfdi:Concepto")
  end
end
